#!/usr/bin/env python
"""
Simple handler for fish layback.

How far behind the ship is the fish?  This is not always an easy
problem to solve.  Hopefully the code here will help out.  There are
different methods for different available data.

The Driscoll xstar chirp rev 1 does not have a depth sensor and the
winch does not have a calble counter.  Even if it did, not all cable
counters work very well.

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: layback.py,v 1.2 2005/10/05 19:24:58 schwehr Exp $

"""

import math
from projections import deg2rad


def simpleLayback (fishdepth, cableout, cableondeck=0., gpsoffset=0., cableconst=1.0):
    """
    Calculate a simple layback with know fishdepth and cable out.

    fishdepth   - meters below sea surface of the fish
    cableout    - meters of cable out from the winch
    cableondeck - distance from winch to block on the a-frame where cable goes down.
                  make this zero if you zero the winch with the cable out to the a-frame
    gpsoffset   - distance from gps antenna to the winch.
                  zero this if the gps knows where the winch is located and make that
                  the origin of the gps measurements.
    cableconst  - Some people make this 0.9 to account for wire sag in the water

    returns     - meters from the gps measured position to the fish location

    This code makes all kinds of assumptions.

      * The fish is directly behind the boat
      * The depth measurement is ok (can be from a sensor or multiple in the data)
      * and I am sure there are others
      
    """
    cableinwater = (cableout - cableondeck) * cableconst

    # a^2 + b^2 = c^2 right triangle
    wateroffset = math.sqrt (cableinwater*cableinwater - fishdepth*fishdepth)

    layback = gpsoffset + cableondeck + wateroffset

    return layback


def angledepthLayback(fishdepth, cableangle=None, speed=3.5, cableondeck=0., gpsoffset=0.):
    """
    Find the layback if you know the cable angle and fishdepth, but not wireout

    fishdepth   - meters below sea surface of the fish
    cableangle  - down angle (plunge) of the wire from the ship to fish (in degrees)
    speed       - ship speed in knots
    cableondeck - distance from winch to block on the a-frame where cable goes down.
                  make this zero if you zero the winch with the cable out to the a-frame
    gpsoffset   - distance from gps antenna to the winch.
                  zero this if the gps knows where the winch is located and make that
                  the origin of the gps measurements.

    returns
    layback - meters from the gps measured position to the fish location
    """

    if None == cableangle:
        if (3.5 != speed):
            sys.exit("ERROR: need to write some sort of cable model for different speeds")
        print "WARNING: guessing what the cable angle is... 40 degrees sounds good"
        cableangle = 40 # FIX: Do something intelligent with ships speed

    angle = deg2rad(cableangle)
    
    wateroffset = fishdepth / math.tan(angle)
    #wireout = fishdepth / math.sin(angle)

    layback = cableondeck + gpsoffset + wateroffset

    return layback
