#!/usr/bin/env python

"""
Module to read segy files.  No writing at this time.

To test: './segy.py -v'


     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: segy.py,v 1.48 2005/10/03 06:07:55 schwehr Exp $

Eventually, each segy version (e.g. xstar/edgetech, and other
instrument tweaks) will be loaded on the fly.  So if a field is
somewhere else, it will be easy to tweak a new config in just a few
minutes

"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.48 $' # See man ident
__date__ = '$Date: 2005/10/03 06:07:55 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'
__credits__ = """Lots of people for helping me with my thesis.
Especially Neal Driscoll and Lisa Tauxe!
"""

import mmap
import os
import struct	# Convert two and from binary formats
import imp	# import modules explicitly
import unittest


BOMBASTIC= 4
VERBOSE  = 3
TRACE    = 2
TERSE    = 1
ALWAYS   = 0


#traceHdrSize = 240 # bytes

# These are the format characters for integers with the struct module for binary conversions
# Force big endian (default on osx, right?)
structIntCodes = { 1: '>b', 2: '>h', 4: '>i', 8: '>q' }


######################################################################
def createDriverName(drvStr):
    """
    Take the user supplied driver name and produce the appropriate
    file to load

    xstar             -> segy_drv_xstar.py
    drv_xstar         -> segy_drv_xstar.py
    segy_drv_xstar    -> segy_drv_xstar.py
    segy_drv_xstar.py -> segy_drv_xstar.py

    FIX: This function needs to be made much more robust.  Maybe try
    multiple find_module calls until module is found
    """
    newStr = drvStr
    #if -1 == newStr.find('.py'):
    #    newStr = newStr+'.py'

    # For imp.find_module we do not want the .py
    if -1 != newStr.find('.py'):
        newStr = newStr[:-3]
    if -1 == newStr.find('drv_'):
        newStr = 'segy_drv_'+newStr
    if -1 == newStr.find('segy_'):
        newStr = 'segy_'+newStr
    return newStr

def getDriverModule(drvStr='segy_drv_rev1.py'):
    """ Load up the module for the segy driver

    Dynamically load the driver for the version of segy
    /sw/share/doc/python24/html/lib/module-imp.html
    """
    drivername = createDriverName(drvStr)
    file, pathname, description = imp.find_module(drivername)
    drv = imp.load_module(drivername,file,pathname,description)

    return drv


######################################################################
def sqlCreateFileTable(specificKeys=None, table='segyFile', drivername='segy_drv_rev1'):
    """
    Build an SQL string to create a table for the keys/fields in a
    SEGY file header

    arguments:
    specificKeys - in set, these are the keys that will used in the create string
    table - SQL table name (may have different tables for different formats)

    FIX: distinguish between 1,2,and 4 byte INTEGERS in the header.
    FIX: CorrelatedTraces and BinGainRecovered are bools, but I treat them as ints
    FIX: maybe check self.drv if this function is provided by the driver.
    """

    drv = getDriverModule(drivername)

    if not specificKeys:
        specificKeys = drv.binaryHeaderEntries.keys()
    cStr  = 'CREATE TABLE '+table+' ('
    cStr += ' fileKey INTEGER PRIMARY KEY'
    cStr += ' , filename VARCHAR(255), '
    for key in specificKeys:
        cStr += ' '+key+' INTEGER,'  # FIX: type needs to change!
    cStr = cStr[:-1]
    cStr += ');'
    return cStr


def sqlCreateTraceTable(specificKeys=None, table='segyTrace', drivername='segy_drv_rev1'):
    """
    Build an SQL string to create a table for the keys/fields in a
    SEGY trace header

    arguments:
    specificKeys - in set, these are the keys that will used in the create string
    table - SQL table name (may have different tables for different formats)

    FIX: distinguish between 1,2,and 4 byte INTEGERS in the header.
    FIX: CorrelatedTraces and BinGainRecovered are bools, but I treat them as ints
    FIX: maybe check self.drv if this function is provided by the driver.
    """
    drv = getDriverModule(drivername)
    
    if not specificKeys:
        specificKeys = drv.traceHeader.keys()

    cStr  = 'CREATE TABLE '+table+' ('
    cStr += ' traceKey INTEGER PRIMARY KEY' # Unique for all traces - autogenerated
    cStr += ' , fileKey INTEGER,' # The file name that contains this trace
    cStr += ' traceNumber INTEGER,' # Trace position inside of the file
    cStr += ' wbt REAL,' # FIX: hack since sqlite 2 has no ALTER TABLE command

    for key in specificKeys:
        cStr += ' '+key+' INTEGER,'
    cStr = cStr[:-1]
    cStr += ');'
    return cStr



######################################################################
class Segy:
    """
    Base class that contains the SEGY header and all the traces

    http://seg.org/publications/tech-stand/
    Then look at segy rev 0 and rev 1
    """

    def __init__(self,filename=None,drivername='segy_drv_rev1', verbosity=0):
        """
        Open up a segy file for reading using mmap for speed.

        filename - name of the segy file to read
        drivername - what format of segy does this file need.
        	FIX: Eventually a value of None should allow you to have this
                code identify the version.
        verbosity - how much spewage
        """
        import imp
        self.verbosity = verbosity
        self.drivername = drivername # FIX: Would like to save this in the db
        self.filename   = filename   # FIX: would like to save this in the db
        self.drv = getDriverModule(drivername)
        
        tmpFile = open(filename,"r+")
        self.size = os.path.getsize(filename)
        self.data = mmap.mmap(tmpFile.fileno(),self.size,access=mmap.ACCESS_READ)
        tmpFile.close() # No longer need the file handle
        self.header=SegyHdr(self.data,self.size,drv=self.drv)

        self.warnedNoSize=False
        return

    def getHeader(self):
        return self.header

    def getTraceHdr(self,traceNumber=1):
        # Save some computation if just the first trace and give something useful for corrupted files
        if 1==traceNumber:
            return SegyTraceHdr(segyHdr=self.header,mmapData=self.header.data,startOffset=3600,drv=self.drv)
        extTextSize = 0 # FIX!!!
        traceStart = 3600 + extTextSize + (traceNumber-1)*self.getTraceSize()
        return SegyTraceHdr(segyHdr=self.header,mmapData=self.header.data,startOffset=traceStart, drv=self.drv)

    def getRawTrace(self,traceNumber):
        """ Return a block of bytes for a trace including header and data"""
        start = self.getTraceStart(traceNumber)
        traceHeaderSize = 240
        end = start + self.getTraceDataSize(traceNumber) + traceHeaderSize
        if self.verbosity>=BOMBASTIC: print 'getRawTrace returning ', start, end
        return (self.data[start:end])

    def getTraceStart(self,traceNumber):
        """
        Return the starting offset in a file of a particular trace
        
        FIX: cope with extended text headers and variable length traces
        """
        extTextSize = 0 # FIX
        traceStart = 3600 + extTextSize + (traceNumber-1)*self.getTraceSize()
        return traceStart


    def getTraceDataSize(self,traceNumber=1):
        """ FIX: handle variable length traces"""
        assert(traceNumber>0) # Start counting traces from 1???
        format = self.header.getBinEntry('SampleFormat')
        f = self.drv.dataFormatStruct[format]

        size = None
        # THIS IS AN UGLY HACK!
        if format==1:
            size = 4 # IBM floating point 4 byte
        else:
            try:
                size = struct.calcsize(f)
            except TypeError:
                # What size is a type 6 with the Eel UTexas data?
                if not self.warnedNoSize:
                    # FIX: find out what these are
                    print "DANGER!!!  Don't know the type size.  Going for 4 bytes"
                    self.warnedNoSize = True
                size = 4
        
        samples = self.header.getBinEntry('SamplesPerTrace')
        datasize = size * samples
        return datasize

    def getTraceSize(self, traceNumber=1):
        """ returns the number of bytes for the particular trace

        fix: cope with variable length traces
        """
        assert (traceNumber>0)
        return 240 + self.getTraceDataSize()

    def getNumberOfTraces(self):
        """ FIX: deal with variable length traces """
        extTextSize = 0 # FIX
        tracesSize = self.size - 3600 - extTextSize
        numTraces = tracesSize / self.getTraceSize()
        if self.verbosity >= BOMBASTIC:
            print "getNumberOfTraces: "
            print "   extTextSize: ",extTextSize
            print "   self.size  : ",self.size
            print "   tracesSize : ",tracesSize
            print "   numTraces  : ",numTraces
            print "   remainder  : ",tracesSize % self.getTraceSize() 
        if 0 != (tracesSize % self.getTraceSize()):
            print "WARNING: seems you have variable length traces that will not be handled!!!"
            assert(False)
        return numTraces
    def getTraceData(self,traceNumber=1):
        """ return a list of samples

        FIX: cope with variable length traces
        """
        extTextSize = 0 # Calc this!
        traceStart = 3600 + extTextSize + (traceNumber-1)*self.getTraceSize()
        #print "traceStart = ", traceStart, traceNumber, self.getTraceSize(), (traceNumber-1)*self.getTraceSize()

        startData = 240 + traceStart
        endData = startData+self.getTraceDataSize()

        format = self.header.getBinEntry('SampleFormat')
        formatStr = self.drv.dataFormatStruct[format]

        if 2==len(formatStr):
            parseStr = formatStr[0] + formatStr[1] * self.header.getBinEntry('SamplesPerTrace')
        elif 1==len(formatStr):
            # FIX: check that floats do not have endian????
            print "FIX: Your data may be bad!  Check to see if we are trying to read a float"
            parseStr = formatStr[0] * self.header.getBinEntry('SamplesPerTrace')
        else:
            print "ERROR: only know how do deal with format strings of length 1 or 2"
            assert(False)
        rawData = self.data[startData:endData]

        traceData = struct.unpack(parseStr,rawData)

        return traceData

    def writeTraceToFile(self,filename='default.dat',traceNumber=1):
        """ Write out pairs of samplenumber and value to a file
        """
        data = self.getTraceData(traceNumber)
        o = open(filename,"w")
        for i in range(0,len(data)):
            o.write(str(i)+" "+str(data[i])+"\n")
        o.close()



        
######################################################################
class SegyHdr:
    """  class for reading segy seismic file header

    http://seg.org/publications/tech-stand/
    Then look at segy rev 0 and rev 1
    """
    def __init__(self,mmapData=None,size=None, filename=None,drv=None):
        """
        Open up a segy file for reading using mmap for speed.
        """
        assert(drv)
        self.drv = drv
        if filename and (not (mmapData and size)):
            tmpFile = open(filename,"r+")
            self.size = os.path.getsize(filename)
            self.data = mmap.mmap(tmpFile.fileno(),self.size,access=mmap.ACCESS_READ)
            tmpFile.close() # No longer need the file handle
            return
        assert(mmapData and size)
        self.data=mmapData
        self.size=size
        return

    def getEntireRawHdr(self):
        """ Return a block of memory containing all of the text, binary, and extended headers"""
        # FIX: cope with extended text file headers

        # Text block + binary header
        hdrlen = (3200+400)
        return (self.data[:hdrlen])

    def getTextFileBlock(self):
        """Return the raw text header block.  Could be either ebcdic or ascii"""

        # FIX: make sure that  this range is not off by 1
        return(self.data[:3200])

    # FIX: move this to Segy class
    def hasR1TextFileHeader(self):
        """Turn True if it has a R1 header with C1..C40 sections as ASCII"""
        if self.data[0:2] == 'C1' or self.data[0:3] == 'C 1':
            return True
        return False

    # FIX: move this to Segy class
    def getTextHeaderLine(self,lineNumber):
        """ Return the string for the line number """
        line = self.data[(lineNumber-1)*80:(lineNumber)*80]
        nul = line.find("\0")
        if -1 == nul: return line
        return line[0:nul]
    
    # FIX: move this to Segy class  or make a text header class
    def getTextHeaderField(self, fieldName):
        """
        BROKEN

        e.g.:
        
        C2  LINE:5t6         AREA:Goleta
        """
        print "FIX: broken"
        entry = self.drv.textFileHeaderEntries[fieldName]
        line = self.getTextHeaderLine(entry[0])
        if len(line) < entry[1]:
            print "Field "+fieldName+" not present"
            return None
        print "FIX: figure out the end of the entry"
        
        return line[entry[1]:]
        
    def getBinEntry(self,fieldName):
        """ Retrieve the value from one of the binary header entries by name """
        assert(0<len(fieldName))

        entry  = self.drv.binaryHeaderEntries[fieldName]
        start  = entry[0] - 1
        end    = entry[1] - 1 
        length = entry[1]-entry[0] + 1
        if length > 8:
            print "get bin header... too large.  skipping ",fieldName
            return None
        val = struct.unpack(structIntCodes[length],self.data[start:end+1])[0]

        return val

    def getTextTranslateValue(self,key,val):
        """ Lookup in the driver what the translation for a value should be

        e.g. 'SampleFormat' of 1 will return:
        4-byte IBM floating point

        """
        table = self.drv.fileHeaderTables[key]
        return table[val]

    def printBinaryHeaders(self, shortList=False, filename=None):
        """ Dump all header into to stdout
        shortList - use the shorter list provided by the driver.  most interesting fields
        """
        if shortList:
            for key in self.drv.fileHeaderShortList:
                val = self.getBinEntry(key)
                if filename: print filename+': ',
                print key+"="+str(val)
            return
            
        for key in self.drv.binaryHeaderEntries.keys():
            val = self.getBinEntry(key)
            if filename: print filename+': ',
            print key+"="+str(val)

        if filename: print filename+': ',
        #print "FIX:",self.drv.DRIVER_NAME, " self.drv.fileHeaderTables.keys()",self.drv.fileHeaderTables.keys()
        for key in self.drv.fileHeaderTables.keys():
            try:
                print key+'=\''+self.getTextTranslateValue(key,self.getBinEntry(key))+"'"
            except KeyError:
                # FIX: handle if there is no key... which means this is a bad driver
                print key+"= Unknown value - '"+str(self.getBinEntry(key))+"'" 

    def sqlInsert(self,filename,specificKeys=None, table='segyFile'):
        """
        Build an SQL string to INSERT the values for the current file

        SEE CREATE...
        FIX: CorrelatedTraces and BinGainRecovered are bools, but I treat them as ints
        FIX: maybe check self.drv if this function is provided by the driver.
        """
        if not specificKeys:
            specificKeys = self.drv.binaryHeaderEntries.keys()
        iStrStart = 'INSERT INTO '+table+' ('
        iStrEnd   = ' VALUES (';

        iStrStart += 'filename,'
        iStrEnd   += "'"+filename+"',"
        for key in specificKeys:
            iStrStart += key+','
            iStrEnd   += str(self.getBinEntry(key))+','
        iStrStart = iStrStart[:-1] + ')'
        iStrEnd   = iStrEnd[:-1] + ');'
        return iStrStart + iStrEnd

######################################################################
# Trace parsing
######################################################################
class SegyTraceHdr:
    """
    Handle one trace in a segy file.
    """
    def __init__(self, segyHdr, startOffset,mmapData=None, drv=None):
        """
        Load up a trace from an mmap'ed data file by jumping to the startOffset
        """
        assert(drv)
        self.drv = drv
        self.data = mmapData
        self.start = startOffset
        self.segyHdr = segyHdr  # FIX: remove this if possible
        return

    def getTextTranslateValue(self,key,val):
        """ Lookup in the driver what the translation for a value should be

        e.g. 'TraceId' 

        """
        table = self.drv.traceHeaderTables[key]
        return table[val]

    # FIX: make both file and trace header table lookups be either Header or entry, but NOT BOTH!
    def getHeader(self,fieldName):
        """Return the interpreted value for a trace header field
        Should know about IBM floats but does not!
        """
        entry  = self.drv.traceHeader[fieldName]
        start  = entry[0] - 1 + self.start
        end    = entry[1] - 1  + self.start
        length = entry[1]-entry[0] + 1
        if length > 8:
            print "get bin header... too large.  skipping ",fieldName
            return None

        if len(entry)>=4:
            """ Means we have a specified data type like IEEEfloat
            e.g.
            'Thickness' : [r2b(50),r2bEnd(50),'Unit thickness for Becca (in seconds)', IEEEfloat],

            """
            # FIX: Make this a lookup table!
            #print "FIX:here? ", len(entry)
            #print "FIX:     ", entry[3]
            code = None
            if 'IEEEfloat' == entry[3]:
                code = '>f' # http://www.python.org/doc/current/lib/module-struct.html
            assert(code)
            #print "FIX: ",code,start,end+1
            val = struct.unpack(code,self.data[start:end+1])[0]
        else:
            try:
                val = struct.unpack(structIntCodes[length],self.data[start:end+1])[0]
            except KeyError:
                print "ERROR with key: ",fieldName, "  Length not found in table",length
                assert(False)

        return val

    def printHeader(self, shortList=False,filename=None,traceNum=None):
        """Dump all trace header info to stdout

        shortList - use the driver specified shorter list of more interesting fields
        """
        if shortList:
            for key in self.drv.traceHeaderShortList:
                val = self.getHeader(key)
                if filename: print filename+': ',
                if traceNum: print str(traceNum)+': ',
                print key+"="+str(val)
            return

        for key in self.drv.traceHeader.keys():
            val = self.getHeader(key)
            if filename: print filename+': ',
            if traceNum: print str(traceNum)+': ',
            print key+"="+str(val)

        # Print out the ascii versions
        for key in self.drv.traceHeaderTables.keys():
            try:
                text = self.getTextTranslateValue(key,self.getHeader(key))
            except:
                text = '__UNKOWN__'
            if filename: print filename+': ',
            if traceNum: print str(traceNum)+': ',
            print key+'_ascii=\''+text+"'"
	return

    def sqlInsert(self,specificKeys=None, table='segyTrace', traceNumber=-1, fileKey=-1):
        """
        Build an SQL string to INSERT the values for the current trace

        SEE CREATE...
        FIX: CorrelatedTraces and BinGainRecovered are bools, but I treat them as ints
        FIX: maybe check self.drv if this function is provided by the driver.


        traceNumber - a trace does not know its number.  Set it if you
        	know it!  the traceNumber is which trace position in
        	the header.  Not necessarily shot number!!!

        FIX FIX FIX: deal with things that are not integers!!!
        """
        if not specificKeys:
            specificKeys = self.drv.traceHeader.keys()
        iStrStart = 'INSERT INTO '+table+' ('
        iStrEnd   = ' VALUES (';

        iStrStart += ' fileKey, ' # for joining with the file table
        iStrEnd   += str(fileKey)+','

        iStrStart += ' traceNumber, '
        iStrEnd   += str(traceNumber)+','
        
        for key in specificKeys:
            iStrStart += key+','
            iStrEnd   += str(self.getHeader(key))+','
        iStrStart = iStrStart[:-1] + ')'
        iStrEnd   = iStrEnd[:-1] + ');'
        return iStrStart + iStrEnd


######################################################################
# Test code

class TestSegy(unittest.TestCase):
    """
    Run the segy class through its paces

    Use one of the BPSIO xstar segy files.  These are not proper segy
    """
    def setUp(self):
        self.xstar = Segy(filename="bpsio01L00.xstar")
        self.ol = Segy(filename="9315MIG.sgy")
        return

    def tearDown(self):
        #print "tearDown: in the end of the test case"
        return

    def testIntro(self):
        """Test the first few bytes of the file - magic number C1 or C 1 """
        #print self.xstar.getTextHeaderLine(1)
        #print self.xstar.getTextHeaderLine(2)
        #print 'CLIENT: ',self.xstar.getTextHeaderField('CLIENT')
        #print 'AREA: ',self.xstar.getTextHeaderField('AREA')

        #f = self.xstar.getBinEntry('SampleFormat')
        #print f,"==",dataFormats[f]

        #self.xstar.printBinaryHeaders()
        
        #print "------------------------------"
        #self.ol.printBinaryHeaders()
        
        self.failUnless(self.xstar.header.hasR1TextFileHeader())
        self.failUnless(not self.ol.header.hasR1TextFileHeader())
        return

    def testSegyTrace(self):
        """ work on the trace class """
        import imp
        drivername = 'segy_drv_rev1'
        drv = getDriverModule(drivername)
        
        # Load up the 1st trace... assume no ext text block
        t = SegyTraceHdr(segyHdr=self.xstar,mmapData=self.xstar.data,startOffset=3600, drv=drv)
        #t.printHeader()
        #key='TraceId'; print key,': ',self.drv.traceIdCode[t.getHeader(key)]

        # look!  If we divide by 3600, we get degrees!
        x = t.getHeader('X'); print x, x/3600.
        y = t.getHeader('Y'); print y, y/3600.
        # e.g.
        # -432364 -->  -120.101111111
        #  123862 -->    34.4061111111

        #print "------------------------------\n  TRACE 1 OL \n"
        #t = SegyTrace(self.ol.data,3600) # Load up the 1st trace... assume no ext text block
        #t.printHeader()

        return


    def testSegyTrace2(self):
        """ work on the trace class - load in two and make sure they are different"""
        segy = Segy('bpsio01L00.xstar')
        t = segy.getTraceHdr()
        print "CHECK: tw getTraceHdr() start:", t.start
        key='Shotpoint'; print key,': ',t.getHeader(key)

        # look!  If we divide by 3600, we get degrees!
        x = t.getHeader('X'); print x, x/3600.
        y = t.getHeader('Y'); print y, y/3600.

        t2 = segy.getTraceHdr(1000)
        print "CHECK: tw getTraceHdr(100) start:", t2.start
        key='Shotpoint'; print key,': ',t2.getHeader(key)

        x2 = t2.getHeader('X'); print x, x/3600.
        y2 = t2.getHeader('Y'); print y, y/3600.

        self.failUnless ( x != x2)
        self.failUnless ( y != y2)
        return


    def testA001Open(self):
        """ Make sure we can open a file """
        segy = Segy('bpsio01L00.xstar')
        self.failUnless(segy)
        return

    def testA002FileBinHeader(self):
        """Be able to read a value out of the file header"""
        segy = Segy('bpsio01L00.xstar')
        hdr = segy.getHeader()
        val = hdr.getBinEntry('JobId')
        print 'JobId = ',val
        self.failUnless(val>=0)
        return

    def testA003KeyValueTranslation(self):
        """ Can we get the english translation of a value?"""
        segy = Segy('bpsio01L00.xstar')
        hdr = segy.getHeader()
        key = 'SampleFormat'
        #print key,'  =  ',hdr.getTextTranslateValue(key,1)
        self.failUnless(hdr.getTextTranslateValue(key,1),'4-byte IBM floating point')
        return

    def testA0004TraceHdr(self):
        """ Read some header fields out of a trace """
        segy = Segy('bpsio01L00.xstar')
        t = segy.getTraceHdr(1)
        x = t.getHeader('X'); print x, x/3600.
        y = t.getHeader('Y'); print y, y/3600.
        return

    def testA0005TraceHdr100(self):
        """ Read some header fields out of a trace """
        segy = Segy('bpsio01L00.xstar')
        t = segy.getTraceHdr(100)
        x = t.getHeader('X'); print x, x/3600.
        y = t.getHeader('Y'); print y, y/3600.
        return

    def testA0006TraceWrite(self):
        segy = Segy('bpsio01L00.xstar')
        segy.writeTraceToFile('1.dat',traceNumber=1)
        segy.writeTraceToFile('2.dat',traceNumber=2)
        segy.writeTraceToFile('3.dat',traceNumber=3)
        segy.writeTraceToFile('100.dat',traceNumber=100)
        return

    def testA0007TracePrintHdr(self):
        segy = Segy('bpsio01L00.xstar')
        t = segy.getTraceHdr(100)
        t.printHeader()
        return

    def testA0010FileSqlCreate(self):
        """Make sure the CREATE TABLE string looks ok"""
        print sqlCreateFileTable()
        return

    def testA0011FileSqlInsert(self):
        """Make sure the INSERT INTO string looks ok"""
        filename='bpsio01L00.xstar'
        segy = Segy(filename)
        print segy.header.sqlInsert(filename)
        return

    def testA0012FileSqlite(self):
        """Make sure the CREATE and INSERT actually pass sqlite"""
        dbName = 'test.db'
        if os.access(dbName,os.W_OK):
            os.remove(dbName)
        filename = 'bpsio01L00.xstar'
        segy = Segy(filename)
        if True: # Dummy scope
            import sqlite
            cx = sqlite.connect(dbName)
            cu = cx.cursor()
            cu.execute(sqlCreateFileTable())
            cu.execute(segy.header.sqlInsert(filename=filename))
            
            cx.commit()
        os.remove(dbName)
        return

    def testA0020FileSqlCreate(self):
        """Make sure the CREATE TABLE string looks ok"""
        print sqlCreateTraceTable()
        return

    def testA0021FileSqlInsert(self):
        """Make sure the INSERT INTO string looks ok"""
        segy = Segy('bpsio01L00.xstar')
        t = segy.getTraceHdr(100)
        print t.sqlInsert()
        return

    def testA0022FileSqlite(self):
        """Make sure the CREATE and INSERT actually pass sqlite"""
        dbName = 'test.db'
        if os.access(dbName,os.W_OK):
            os.remove(dbName)
        segy = Segy('bpsio01L00.xstar')
        traceNumber=100
        t = segy.getTraceHdr(traceNumber)
        if True: # Dummy scope
            import sqlite
            cx = sqlite.connect(dbName)
            cu = cx.cursor()
            cu.execute(sqlCreateTraceTable())
            cu.execute(t.sqlInsert(traceNumber=traceNumber))
            
            cx.commit()
        os.remove(dbName)
        return

    def testA0000SqlGlobalCreate(self):
        """Make sure the globally scoped create database functions work"""
        createFileTable  = sqlCreateFileTable()
        createTraceTable = sqlCreateTraceTable()
        print 'createFileTable  = ',createFileTable,'\n'
        print 'createTraceTable = ',createTraceTable,'\n'

######################################################################
class TestDriverName(unittest.TestCase):
    def test01Nop(self):
        self.failUnless('segy_drv_xstar.py'==createDriverName('segy_drv_xstar.py'))
    def test02Py(self):
        self.failUnless('segy_drv_xstar.py'==createDriverName('segy_drv_xstar'))
    def test03drv(self):
        self.failUnless('segy_drv_xstar.py'==createDriverName('xstar'))
    def test04segy(self):
        self.failUnless('segy_drv_xstar.py'==createDriverName('drv_xstar'))
    def test05(self):
        self.failUnless('segy_drv_xstar.py'==createDriverName('drv_xstar.py'))
    def test06(self):
        self.failUnless('segy_drv_xstar.py'==createDriverName('xstar.py'))
    def test07(self):
        self.failUnless('segy_drv_rev1.py'==createDriverName('rev1'))
    def test10load(self):
        drv = getDriverModule('xstar')
        self.failUnless('xstar'==drv.DRIVER_NAME)

########################################
# Run the test code
if __name__ == '__main__':
    print "Testing segy"

    #import doctest, segy
    #doctest.testmod(segy)
    unittest.main()

